@@ -6,7 +6,7 @@ no  warnings 'uninitialized';
 
 use vars qw($VERSION);
 
-$VERSION = '1.44';
+$VERSION = '1.45';
 
 =head1 NAME
 
@@ -14,7 +14,7 @@ Yote - Persistant Perl container objects in a directed graph of lazilly loaded n
 
 =head1 DESCRIPTION
 
-This is for anyone who wants to store arbitrary structured state data and doesn't have 
+This is for anyone who wants to store arbitrary structured state data and doesn't have
 the time or inclination to write a schema or configure some framework. This can be used
 orthagonally to any other storage system.
 
@@ -24,8 +24,8 @@ container is a key/value store where the values can be strings, numbers, arrays,
 or other Yote containers.
 
 The entry point for all Yote data stores is the root node. All objects in the store are
-subject to recycling if they cannot trace a reference path back to this node. Recycling
-is called manually.
+unreachable if they cannot trace a reference path back to this node. If they cannot, running
+compress_store will remove them.
 
 There are lots of potential uses for Yote, and a few come to mind :
 
@@ -38,30 +38,30 @@ There are lots of potential uses for Yote, and a few come to mind :
  * product information
 
 =head1 SYNOPSIS
- 
+
  use Yote;
 
  my $store = Yote::open_store( '/path/to/data-directory' );
 
  my $root_node = $store->fetch_root;
 
- $root_node->add_to_myList( $store->newobj( { 
+ $root_node->add_to_myList( $store->newobj( {
     someval  => 123.53,
     somehash => { A => 1 },
-    someobj  => $store->newobj( { foo => "Bar" }, 
+    someobj  => $store->newobj( { foo => "Bar" },
                 'Optional-Yote-Subclass-Package' );
  } );
 
- # the root node now has a list 'myList' attached to it with the single 
- # value of a yote object that yote object has two fields, 
+ # the root node now has a list 'myList' attached to it with the single
+ # value of a yote object that yote object has two fields,
  # one of which is an other yote object.
- 
+
  $root_node->add_to_myList( 42 );
 
  #
  # New Yote container objects are created with $store->newobj. Note that
  # they must find a reference path to the root to be protected from
- # recycling. 
+ # being deleted from the record store upon compression.
  #
  my $newObj = $store->newobj;
 
@@ -110,14 +110,8 @@ There are lots of potential uses for Yote, and a few come to mind :
 
  #
  # Anything that cannot trace a reference path to the root
- # is eligable for being recycled. A recycled object's ID
- # will be reused for new objects.
+ # is eligable for being removed upon compression. 
  #
- $myList->[0]->set_condition( "About to be recycled" );
- delete $myList->[0];
-
- $store->stow_all;
- $store->run_recycler;
 
 =head1 PUBLIC METHODS
 
@@ -127,8 +121,9 @@ There are lots of potential uses for Yote, and a few come to mind :
 =head2 open_store( '/path/to/directory' )
 
 Starts up a persistance engine and returns it.
-   
+
 =cut
+
 sub open_store {
     my $path = pop;
     my $store = Yote::ObjStore->_new( { store => $path } );
@@ -146,6 +141,8 @@ no warnings 'numeric';
 no warnings 'uninitialized';
 no warnings 'recursion';
 
+use Devel::Refcount 'refcount';
+use File::Copy;
 use WeakRef;
 use Module::Loaded;
 
@@ -161,7 +158,7 @@ The Yote::ObjStore does the following things :
  * creates new objects
  * fetches existing objects by id
  * saves all new or changed objects
- * finds objects that cannot connect to the root node and recycles them
+ * finds objects that cannot connect to the root node and removes them
 
 =cut
 
@@ -170,10 +167,10 @@ The Yote::ObjStore does the following things :
 # ------------------------------------------------------------------------------------------
 
 =head2 fetch_root
-    
- Returns the root node of the graph. All things that can be 
+
+ Returns the root node of the graph. All things that can be
 trace a reference path back to the root node are considered active
-and are not recycled when the recyler run.
+and are not removed when the object store is compressed.
 
 =cut
 sub fetch_root {
@@ -190,14 +187,14 @@ sub fetch_root {
 
 =head2 newobj( { ... data .... }, optionalClass )
 
- Creates a container object initialized with the 
+ Creates a container object initialized with the
  incoming hash ref data. The class of the object must be either
  Yote::Obj or a subclass of it. Yote::Obj is the default.
 
- Once created, the object will be saved in the data store when 
- $store->stow_all has been called.  If the object is not attached 
- to the root or an object that can be reached by the root, it will be 
- recycled when Yote::Obj::recycle_objects is called.
+ Once created, the object will be saved in the data store when
+ $store->stow_all has been called.  If the object is not attached
+ to the root or an object that can be reached by the root, it will be
+ remove when Yote::ObjStore::Compress is called.
 
 =cut
 sub newobj {
@@ -206,7 +203,7 @@ sub newobj {
     $class->_new( $self, $data );
 }
 
-sub _newroot { 
+sub _newroot {
     my $self = shift;
     Yote::Obj->_new( $self, {}, $self->_first_id );
 }
@@ -267,17 +264,95 @@ sub fetch {
 
 =head2 run_recycler
 
-Does a mark and sweep and recycles all objects that
-do not currently have an active reference or can 
-trace a reference path back to the root object.
-
 =cut
 sub run_recycler {
     my $self = shift;
-    $self->stow_all();
+     $self->stow_all();
     $self->{_DATASTORE}->_recycle_objects();
 } #run_recycler
 
+=head2 compress_store
+
+This creates a new data store with all objects
+removed that cannot be traced back to the root.
+
+This will fail if the following directories 
+exist :
+  ${ROOT-DIRECOTRY-FOR-OBJECT-STORE}_COMPRESS_BAK
+  ${ROOT-DIRECOTRY-FOR-OBJECT-STORE}_NEW_RECYC
+
+The first directory is where the original object
+store gets backed up after the compression. If 
+the compression is a success, it should be removed.
+
+The second is a temporary working directory for the
+new copied over database.
+
+This requires enough room on the file system
+to potentially double the size of the database
+temporarily as it is copied to a new one.
+
+=cut
+sub compress_store {
+    my( $self, $newdir, $backdir ) = @_;
+    $self->stow_all();
+
+    my $original_dir = $self->{args}{store};
+    $backdir //= $original_dir . '_COMPRESS_BAK';
+    $newdir  //= $original_dir . '_NEW_RECYC';
+
+    if( -x $backdir ) {
+        die "Unable to run compress store, backup directory '$backdir' already exists.";
+    }
+
+    if( -x $newdir ) {
+        die "Unable to run compress store, temp directory '$newdir' already exists.";
+    }
+
+    if( $self->_has_weak_refs ) {
+        die "Unable to run compress store. There are still outstanding references to yote objects that would be deleted during the compress.";
+    }
+    my $newstore = Yote::ObjStore->_new( { store => $newdir } );
+
+    my( @copy_ids ) = ( $self->_first_id );
+
+    my $count = 0;
+    my( %seen );
+    while( @copy_ids ) {
+        my $id = shift @copy_ids;
+#        next if $seen{$id}++;
+        next if $newstore->{_DATASTORE}{DATA_STORE}->has_id( $id ) && $id != $self->_first_id;
+        next if$id == $self->_first_id && $count > 0;
+        
+        print STDERR "\t$id";
+        if( ++$count > 80 ) {
+            print STDERR "\n";
+            $count = 0;
+        }
+        
+        my $obj = $self->fetch( $id );
+#        $obj->{STORE} = $newstore;
+
+        $newstore->{_DATASTORE}{DATA_STORE}->ensure_entry_count( $id - 1 );
+        $newstore->_dirty( $obj, $id );
+        $newstore->_stow( $obj, $id );
+
+        my $r = ref( $obj );
+        if ( $r eq 'ARRAY' ) {
+            my $tied = tied @$obj;
+            push @copy_ids, grep { $_ > 0 } @{$tied->[1]};
+        } elsif ( $r eq 'HASH' ) {
+            my $tied = tied %$obj;
+            push @copy_ids, grep { $_ > 0 } values %{$tied->[1]};
+        } else {
+            push @copy_ids, grep { $_ > 0 } values %{$obj->{DATA}};
+        }
+    }
+    
+    move( $original_dir, $backdir ) or die $!;
+    move( $newdir, $original_dir ) or die $!;
+
+} #compress_store
 =head2 stow_all
 
  Saves all newly created or dirty objects.
@@ -349,8 +424,10 @@ sub _get_id {
 
 sub __get_id {
     my( $self, $ref ) = @_;
+
     my $class = ref( $ref );
     die "__get_id requires reference. got '$ref'" unless $class;
+    
     if( $class eq 'Yote::Array') {
         return $ref->[0];
     }
@@ -405,14 +482,15 @@ sub __get_id {
 } #_get_id
 
 sub _stow {
-    my( $self, $obj ) = @_;
+    my( $self, $obj, $id ) = @_;
 
     my $class = ref( $obj );
     return unless $class;
-    my $id = $self->_get_id( $obj );
+    $id //= $self->_get_id( $obj );
     die unless $id;
 
     my $data = $self->_raw_data( $obj );
+
     if( $class eq 'ARRAY' ) {
         $self->{_DATASTORE}->_stow( $id,'ARRAY', $data );
         $self->_clean( $id );
@@ -486,6 +564,14 @@ sub _is_dirty {
     $ans;
 } #_is_dirty
 
+sub _has_weak_refs {
+    my $self = shift;
+
+    # if there is an ARRAY or HASH, since it is tied, it has a reference to itself
+    0 < grep { $_ && refcount($_) > (ref($_) =~ /^(ARRAY|HASH)$/ ? 1 : 0 ) }
+        values %{$self->{_WEAK_REFS}};
+}
+
 sub _purge {
     my $self = shift;
     $self->{_DIRTY} = {};
@@ -526,14 +612,14 @@ sub _raw_data {
     else {
         return $obj->{DATA};
     }
-    
+
 } #_raw_data
 
 sub _store_weak {
     my( $self, $id, $ref ) = @_;
     die unless $ref;
     $self->{_WEAK_REFS}{$id} = $ref;
-    
+
     weaken( $self->{_WEAK_REFS}{$id} );
 } #_store_weak
 
@@ -603,7 +689,7 @@ sub id {
 
 =head2 set( $field, $value )
 
-    Assigns the given value to the field in this object and returns the 
+    Assigns the given value to the field in this object and returns the
     assigned value.
 
 =cut
@@ -615,7 +701,7 @@ sub set {
         $self->{STORE}->_dirty( $self, $self->{ID} );
     }
 
-    
+
     $self->{DATA}{$fld} = $inval;
     return $self->{STORE}->_xform_out( $self->{DATA}{$fld} );
 } #set
@@ -766,8 +852,8 @@ sub AUTOLOAD {
 # -----------------------
 
 =head2 _init
-  
-    This is called the first time an object is created. It is not 
+
+    This is called the first time an object is created. It is not
     called when the object is loaded from storage. This can be used
     to set up defaults. This is meant to be overridden.
 
@@ -775,7 +861,7 @@ sub AUTOLOAD {
 sub _init {}
 
 =head2 _init
-  
+
     This is called each time the object is loaded from the data store.
     This is meant to be overridden.
 
@@ -833,7 +919,7 @@ sub _DUMP_ALL {
     delete $show->{$self->{ID}};
     $seen->{$self->{ID}} = 1;
     my $buf = $self->_DUMP;
-    
+
     for my $obj_id (sort { $a <=> $b } grep { index($_,'v') != 0 && 0 == $seen->{$_}++ } values %{$self->{DATA}}) {
         $show->{$obj_id} = 1;
     }
@@ -845,7 +931,7 @@ sub _DUMP_ALL {
 
         my $obj = $self->{STORE}->fetch( $obj_id );
         my $r = ref( $obj );
-        
+
         if( $r eq 'ARRAY' ) {
             $buf .= "$obj_id (ARRAY)\n";
             my $tied = tied @$obj;
@@ -1020,13 +1106,13 @@ sub STORE {
     $self->[1]{$key} = $self->[2]->_xform_in( $val );
 }
 
-sub FIRSTKEY { 
+sub FIRSTKEY {
     my $self = shift;
     my $a = scalar keys %{$self->[1]};
     my( $k, $val ) = each %{$self->[1]};
     return wantarray ? ( $k => $val ) : $k;
 }
-sub NEXTKEY  { 
+sub NEXTKEY  {
     my $self = shift;
     my( $k, $val ) = each %{$self->[1]};
     return wantarray ? ( $k => $val ) : $k;
@@ -1120,7 +1206,7 @@ sub _first_id {
 } #_first_id
 
 #
-# Create a new object id and return it. 
+# Create a new object id and return it.
 #
 sub _get_id {
   my $self = shift;
@@ -1133,11 +1219,6 @@ sub _max_id {
   shift->{DATA_STORE}->entry_count;
 }
 
-# used for debugging and testing
-sub _get_recycled_ids {
-  shift->{DATA_STORE}->_get_recycled_ids;
-}
-
 sub _recycle_objects {
   my $self = shift;
 
@@ -1216,15 +1297,16 @@ sub _recycle_objects {
 
   # remove temporary recycle datastore
   $mark_to_keep_store->unlink_store;
-  
-  return $count;
+
+  $self->{DATA_STORE}->_get_recycled_ids;
   
 } #_recycle_objects
 
+
 #
 # Saves the object data for object $id to the data store.
 #
-sub _stow {
+sub _stow { #Yote::YoteDB::_stow
   my( $self, $id, $class, $data ) = @_;
   my $save_data = "$class " . to_json($data);
   $self->{DATA_STORE}->stow( $save_data, $id );
